@@ -4,6 +4,7 @@
 ##' @title update_all
 ##' @param check_R whether check R version
 ##' @param which repo (CRAN, BioC, github) to update
+##' @param lib.loc location of library, default is NULL and will set to .libPaths()
 ##' @return NULL
 ##' @importFrom utils update.packages
 ##' @importFrom utils remove.packages
@@ -14,21 +15,21 @@
 ##' update_all()
 ##' }
 ##' @author Guangchuang Yu
-update_all <- function(check_R=TRUE, which=c("CRAN", "BioC", "github")) {
+update_all <- function(check_R=TRUE, which=c("CRAN", "BioC", "github"), lib.loc = NULL) {
     if (check_R && !check_r()$up_to_date) {
         stop("you need to upgrade your R first...")
     }
 
     if ('CRAN' %in% which) {
-        update_cran()
+        update_cran(lib.loc = lib.loc)
     }
 
     if ('BioC' %in% which) {
-        update_bioc()
+        update_bioc(lib.loc = lib.loc)
     }
 
     if ('github' %in% which) {
-        update_github()
+        update_github(lib.loc = lib.loc)
     }
     message("done....")
 }
@@ -37,19 +38,21 @@ is_bioc_up_to_date <- function() {
     suppressMessages(check_bioc()$up_to_date)
 }
 
-update_cran <- function() {
+update_cran <- function(lib.loc = NULL) {
     message("upgrading CRAN packages...")
-    tryCatch(update.packages(ask=FALSE, checkBuilt=TRUE),
+    tryCatch(update.packages(lib.loc = lib.loc,
+                             ask=FALSE,
+                             checkBuilt=TRUE),
              error=function(e) NULL)
 }
 
 ##' @importFrom utils install.packages
-update_bioc <- function() {
+update_bioc <- function(lib.loc = NULL) {
     pkg <- "BiocManager"
-    bioc_version <- tryCatch(packageVersion(pkg), error=function(e) NULL)
+    bioc_version <- tryCatch(packageVersion(pkg, lib.loc = lib.loc), error=function(e) NULL)
     flag <- "BiocManager"
     if (is.null(bioc_version)) {
-        biocLite <- tryCatch(packageVersion("BiocInstaller"), error=function(e) NULL)
+        biocLite <- tryCatch(packageVersion("BiocInstaller", lib.loc = lib.loc), error=function(e) NULL)
         if (is.null(biocLite)){
             flag <- "No_BioC"
         } else if (check_r()$installed_version < "R-3.5.0") {
@@ -57,8 +60,8 @@ update_bioc <- function() {
         } else {
             message('Bioconductor has switched to a new package manager: "BiocManager".')
             message("Removing BiocInstaller and install BiocManager")
-            remove.packages("BiocInstaller")
-            install.packages("BiocManager")
+            remove.packages("BiocInstaller", lib = lib.loc)
+            install.packages("BiocManager", lib = lib.loc)
             flag <- "BiocManager"
         }
     }
@@ -71,7 +74,7 @@ update_bioc <- function() {
         invisible(readline(prompt="Press [enter] to continue to update Bioconductor (outdated release that fit your R version)"))
         if ("BiocInstaller" %in% loadedNamespaces()) {
             detach("package:BiocInstaller", character.only=TRUE)
-            remove.packages("BiocInstaller")
+            remove.packages("BiocInstaller", lib = lib.loc)
             source("https://www.bioconductor.org/biocLite.R")
         }
         suppressPackageStartupMessages(require(pkg, character.only = TRUE))
@@ -115,7 +118,7 @@ update_bioc <- function() {
 
         message("upgrading BioC packages...")
         suppressPackageStartupMessages(require(pkg, character.only = TRUE))
-        install(ask=FALSE, checkBuilt=TRUE, version = install_version)
+        install(ask=FALSE, checkBuilt=TRUE, version = install_version, lib.loc = lib.loc)
         
     }
 }
@@ -123,15 +126,18 @@ update_bioc <- function() {
 
 ##' @importFrom utils installed.packages
 ##' @importFrom utils packageDescription
-update_github <- function() {
+update_github <- function(lib.loc = NULL) {
     message("upgrading github packages...")
     pkgs <- installed.packages()[, 'Package']
-    install_github <- get_fun_from_pkg("devtools", "install_github")
+    install_github <- get_fun_from_pkg("remotes", "install_github")
     tmp <- sapply(pkgs, function(pkg) {
-        desc <- packageDescription(pkg)
+        desc <- packageDescription(pkg, lib.loc = lib.loc)
         if (length(desc) <= 1 || is.null(desc$GithubSHA1))
             return(NULL)
-        tryCatch(install_github(repo=paste0(desc$GithubUsername, '/', desc$GithubRepo), checkBuilt=TRUE),
+        tryCatch(install_github(repo=paste0(desc$GithubUsername, '/', desc$GithubRepo),
+                                ref = desc$GithubRef,
+                                checkBuilt=TRUE,
+                                lib.loc = lib.loc),
                  error=function(e) NULL)
     })
 }
